"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs = require("fs");
var crypto = require("crypto");
var getStream = require("get-stream");
var request = require("request");
/** return the sha256 of a file as a hex-formatted string */
function sha256OfFile(path) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var input, cipher;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    input = fs.createReadStream(path);
                    cipher = crypto.createHash('sha256');
                    return [4 /*yield*/, getStream(input.pipe(cipher), { encoding: 'hex' })];
                case 1: return [2 /*return*/, _c.sent()];
            }
        });
    });
}
exports.sha256OfFile = sha256OfFile;
/** download a URL and save it to a file */
function downloadUrlToFile(url, path) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var fileStream;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    fileStream = fs.createWriteStream(path);
                    return [4 /*yield*/, new Promise(function (res, rej) {
                            request(url).pipe(fileStream).on('close', function () {
                                res();
                            });
                        })];
                case 1:
                    _c.sent();
                    return [2 /*return*/];
            }
        });
    });
}
exports.downloadUrlToFile = downloadUrlToFile;
function readFileWithDefault(path, defaultContent) {
    try {
        return fs.readFileSync(path, 'utf8');
    }
    catch (e) {
        if (!fs.existsSync(path))
            return defaultContent;
        throw e;
    }
}
exports.readFileWithDefault = readFileWithDefault;
function readJsonFileWithDefault(path, defaultContent) {
    try {
        return JSON.parse(fs.readFileSync(path, 'utf8'));
    }
    catch (e) {
        if (!fs.existsSync(path))
            return defaultContent;
        throw e;
    }
}
exports.readJsonFileWithDefault = readJsonFileWithDefault;
function writeJsonFile(path, value, indent) {
    if (indent === void 0) { indent = true; }
    if (indent === true)
        indent = '    ';
    fs.writeFileSync(path, JSON.stringify(value, null, indent));
}
exports.writeJsonFile = writeJsonFile;
function patchJsonFile(path, _a, _b) {
    var _c = [_a, _b], _d = _c[0], opts = _d === void 0 ? {} : _d, callback = _c[1];
    if (!callback)
        _e = [{}, _a], opts = _e[0], callback = _e[1];
    var _f = opts.defaultValue, defaultValue = _f === void 0 ? null : _f, _g = opts.indent, indent = _g === void 0 ? false : _g;
    var value = readJsonFileWithDefault(path, defaultValue);
    var newValue = callback(value);
    if (newValue === undefined)
        newValue = value;
    writeJsonFile(path, newValue, indent);
    var _e;
}
exports.patchJsonFile = patchJsonFile;
